/**
 * Opens the Document form.
 *
 * formID
 *   Integer index of the form in the document.forms collection that contains the input specified by the
 *   valueFieldName parameter.
 * valueFieldName
 *   Name of the input control that receives the OID of the document entity.
 * displayFieldName
 *   Name of the input control that receives the display value of the document entity. Can be null.
 * doPostback
 *   Boolean that specifies how the calling window should be updated.
 *   If true, a post of the form will be used. If false, the calling window will be refreshed.
 * transID (optional; internal to change tracking)
 * overrideKey (optional; internal to change tracking)
 * showOkAndAddAnother (optional)
 *   Boolean indicating whether to show the "OK and Add Another" button. The default is to not show it.
 * activityType (optional)
 *   If change tracking is to be turned on, this is the OID of the relevant ActivityType entity.
 *   Previously, this parameter was simply a boolean indicating whether change tracking should be
 *   enabled. However, the addition of "interim change tracking" in 5.6 made it necessary to know
 *   more about the activity to be logged.
 * setOID (optional)
 *   When editing an existing set member, this is the OID of the set itself. Passing this causes
 *   a hint to be given to the change tracking system if the transaction commits.
 * rootEntity (optional)
 *   Used by the view validation and binding override script hooks.
 * wizardPage (optional)
 *   If this popup form is being used within the context of a SmartForm, this is the OID of the current
 *   WizardPage. This parameter was added in support of "interim change tracking" in 5.6. The OID
 *   is just added to the querystring for the benefit of Side-by-Side View Differences (custom).
 * aldOID (optional)
 *   OID of the AttributeLayoutDescription entity corresponding to the view control, if applicable.
 *   This argument is optional and is used for ajax postbacks. If the doPostback argument is true and
 *   aldOID is also passed, an ajax postback will be used to update the control. If aldOID is not
 *   specified, a standard postback will be performed.
 */

function showDocumentForm(formID, valueFieldName, displayFieldName, doPostback, transID, overrideKey,
                          showOkAndAddAnother, activityType, setOID, rootEntity, wizardPage, aldOID, versionedResource, viewOid, hideAdvancedButton)
{
	var arrWindowUrl = getDocumentFormUrl(formID, valueFieldName, displayFieldName, doPostback, transID, overrideKey,
                          showOkAndAddAnother, activityType, setOID, rootEntity, wizardPage, aldOID, null, viewOid, hideAdvancedButton);
	var winWidth = 620;
	var winHeight = 300;
	var arrCtrWinPos = _getCenteredWindowPosition(winWidth, winHeight);
	var winFeatures =
		["width=", winWidth, ",height=", winHeight, ",left=", arrCtrWinPos[0], ",top=", arrCtrWinPos[1],
		",resizable=yes,scrollbars=yes,status=no,location=no,menubar=no,toolbar=no"].join("");
	window.open(arrWindowUrl, "_blank", winFeatures);	
}

//Gets the URL to the document form without opening it in a popup
function getDocumentFormUrl(formID, valueFieldName, displayFieldName, doPostback, transID, overrideKey,
                          showOkAndAddAnother, activityType, setOID, rootEntity, wizardPage, aldOID, versionedResource, viewOid, hideAdvancedButton)
{
	var errPrefix = "EXCEPTION WebrCommon/Scripts/EntityChooser/DocumentFormLauncher.js showDocumentForm():\r\n\r\n";
	var arrWindowUrl = [_documentFormURL, "?postback=", (doPostback ? "1" : "0")];
	if (isNaN(formID)) {
		alert(errPrefix + "Argument formID is invalid");
		return;
	}
	arrWindowUrl.push("&formID=", formID);
	if (valueFieldName == null || valueFieldName == "") {
		alert(errPrefix + "Argument valueFieldName is invalid");
		return;
	}
	var valueField = document.forms[formID].elements[valueFieldName];
	if (valueField == null) {
		alert(errPrefix + "Cannot find form field " + valueFieldName);
		return;
	}
	arrWindowUrl.push("&valueField=", escape(valueFieldName));
	
	if (versionedResource != null && versionedResource != ""){
		arrWindowUrl.push("&VersionedResource=", escape(versionedResource));
	}
	else if (valueField.value != null && valueField.value !== "" && valueField.value != ":00000000000000000000000000000000") {
		arrWindowUrl.push("&VersionedResource=", escape(valueField.value));
		// If this is the set control, clear the value in the value field.
		if (valueFieldName.search(/\_setItemAdd$/) != -1) {
			valueField.value = "";
		}
	}
	if (displayFieldName != null && displayFieldName != "") {
		arrWindowUrl.push("&displayField=", escape(displayFieldName));
	}
	if (transID != null && transID != "") {
		arrWindowUrl.push("&_webrChangeTrackingParentTransID=", escape(transID));
	}
	if (overrideKey != null && overrideKey != "") {
		arrWindowUrl.push("&overrideKey=",  escape(overrideKey));
	}
	if (showOkAndAddAnother) {
		arrWindowUrl.push("&showOkAndAddAnother=true");
	}
	if (activityType != null && activityType != "") {
		arrWindowUrl.push("&activityType=", escape(activityType));
	}
	if (setOID != null && setOID != "") {
		arrWindowUrl.push("&setOID=", escape(setOID));
	}
	if (rootEntity != null && rootEntity != "") {
		arrWindowUrl.push("&rootEntity=", escape(rootEntity));
	}
	if (wizardPage != null && wizardPage != "") {
		arrWindowUrl.push("&WizardPageOID=", escape(wizardPage));
	}
	if (aldOID != null && aldOID != "") {
		arrWindowUrl.push("&aldOID=", escape(aldOID));
	}
	if (viewOid != null && viewOid != "") {
		arrWindowUrl.push("&view=", escape(viewOid));
	}
	if (hideAdvancedButton) {
		arrWindowUrl.push("&hideAdvancedButton=true");
	}
	
	return arrWindowUrl.join("");
}


function showHistoryForm(formID, valueFieldName)
{
	var errPrefix = "EXCEPTION WebrCommon/Scripts/EntityChooser/DocumentFormLauncher.js showHistoryForm():\r\n\r\n";
	if (isNaN(formID)) {
		alert(errPrefix + "Argument formID is invalid");
		return;
	}
	if (valueFieldName == null || valueFieldName == "") {
		alert(errPrefix + "Argument valueFieldName is invalid");
		return;
	}
	var valueField = document.forms[formID].elements[valueFieldName];
	if (valueField == null) {
		alert(errPrefix + "Cannot find form field " + valueFieldName);
		return;
	}
	var docOID = valueField.value;
	if (docOID == "" || docOID == ":00000000000000000000000000000000") {
		return;
	}
	var winWidth = 750;
	var winHeight = 600;
	var arrCtrWinPos = _getCenteredWindowPosition(winWidth, winHeight);
	var winFeatures =
		["width=", winWidth, ",height=", winHeight, ",left=", arrCtrWinPos[0], ",top=", arrCtrWinPos[1],
		",resizable=yes,scrollbars=yes,status=no,location=no,menubar=no,toolbar=no"].join("");
	var windowUrl = [_documentHistoryFormURL, "?bindTo=", escape(docOID), "&Document=", escape(docOID)].join("");
	window.open(windowUrl, "_blank", winFeatures);
}

// For a window of the specified dimensions, this returns an array containing the coordinates
// for the top left corner of the window when centered on the screen.
function _getCenteredWindowPosition(iWinWidth, iWinHeight)
{
	var iWinLeft = 0;
	var iWinTop = 0;
	if (typeof window.screen != "undefined") {
		iWinLeft = (window.screen.availWidth - iWinWidth) / 2;
		if (iWinLeft < 0) iWinLeft = 0;
		iWinTop = (window.screen.availHeight - iWinHeight) / 2;
		if (iWinTop < 0) iWinTop = 0;
   	}
   	return [iWinLeft, iWinTop];
}