(function() {
    "use strict";
	
	// provide some protection against this code being initialized twice, which
	// could happen if jQuery loads content via AJAX that references this file
	// in a page that already referenced it
	if(typeof(window.PortalTools) == 'object') {
		// this has already been defined
		return;
	}
	window.PortalTools = {};
	window.DynamicResultSetView = {};
		
	// define some help methods for dealing with empty strings in the data element	
	PortalTools.IsStringNullOrEmpty = function(str) {
		if(str === null || typeof str === "undefined") return true;
		if(typeof str !== "string") throw new Error("Not a string");
		return /^[\s|\u00A0]*$/.test(str);
	};
	
	function filterEmptyStrings(str) {
		return PortalTools.IsStringNullOrEmpty(str) ? null : str;
	}
	
	// load values from the data element
	var portalToolsData = document.getElementById('PortalToolsData').dataset;
	PortalTools.QueryString = null;
	PortalTools.tabState = {};
	PortalTools.store = portalToolsData.store;
	PortalTools.callEntityMethodURL = portalToolsData.callEntityMethodUrl;
	PortalTools.urlPrefix = portalToolsData.urlPrefix;
	PortalTools.webrCommonPath = portalToolsData.webrCommonPath;
	PortalTools.currentResource = filterEmptyStrings(portalToolsData.currentResource);
	PortalTools.currentContainer = filterEmptyStrings(portalToolsData.currentContainer);
	PortalTools.currentResourceID = filterEmptyStrings(portalToolsData.currentResourceId);
	PortalTools.currentContainerID = filterEmptyStrings(portalToolsData.currentContainerId);
	PortalTools.currentUser = filterEmptyStrings(portalToolsData.currentUser);
	PortalTools.busyGifPath = portalToolsData.busyGifPath;
	PortalTools.dropLayoutIndicatorPath = portalToolsData.dropLayoutIndicatorPath;
	PortalTools.timestamp = parseInt(portalToolsData.timestamp, 10);
	PortalTools.monitorEntityMethodURL = portalToolsData.monitorEntityMethodUrl;
	PortalTools.remoteMethodMonitorFirstPollDelay = parseInt(portalToolsData.remoteMethodMonitorFirstPollDelay, 10);
	PortalTools.remoteMethodMonitorPollInterval = parseInt(portalToolsData.remoteMethodMonitorPollInterval, 10);
	PortalTools.debugUtilModuleEnableFlag = portalToolsData.debugUtilModuleEnableFlag === "true";
	PortalTools.newSessionRedirect = portalToolsData.newSessionRedirect;
	PortalTools.DRSVHandlerURL = portalToolsData.drsvHandlerUrl;
	DynamicResultSetView.debug = portalToolsData.drsvDebug === "true";
	DynamicResultSetView.statePersistenceMode = portalToolsData.drsvPersistenceMode;

	// client side code may need to know what integrated stores it should request 
	// data from, so we make a list of integrated stores available
	PortalTools.IntegratedStores = [];
	var cpipDataElements = document.getElementsByClassName('IntegratedStoreData');
	Array.prototype.forEach.call(cpipDataElements, function(ele){
	    var store = {
	    	name: ele.dataset.name,
	    	url: ele.dataset.url,
	    	version: ele.dataset.version
	    };
	    PortalTools.IntegratedStores.push(store);
	});

	PortalTools.RemoteMethod = function(methodName) {
		
		// this are "private" values
		var method = methodName;
		var queryStringValues = {};
		var formDataValues = {};
		var argumentValue = null;
		var spinnerOnly = true;
		var uiProgressBar = null;
		var sourceStore = null;
		var returnObj = {};
		
		// expose accesors for the private values
		returnObj.QueryString = function(newValues) {
			queryStringValues = newValues;
			return this;
		};
		
		returnObj.FormData = function(newValues) {
			formDataValues = newValues;
			return this;
		};
		
		returnObj.Argument = function(newArgument) {
			argumentValue = newArgument;
			return this;
		};
		
		returnObj.ProgressBar = function(newProgressBarElement, onlyUseSpinner) {
			spinnerOnly = onlyUseSpinner;
			uiProgressBar = newProgressBarElement;
			return this;
		};
		
		returnObj.SourceStore = function(newSourceStore) {
			sourceStore = newSourceStore;
			return this;
		};
		
		returnObj.Call = function() {
			var qs = [];
			var timestamp = (new Date()).getTime();
			$.ajaxSetup({ cache: true });
			
			if(typeof sourceStore === "string") {
				qs.push("sourceStore=" + sourceStore);
			}
			qs.push("time=" + timestamp);
        	qs.push("method=" + method);
        	for(var i in queryStringValues) {
        		qs.push(i + "=" + queryStringValues[i]);
        	}
        	
        	formDataValues.time = encodeURIComponent(timestamp.toString());
        	formDataValues.method = encodeURIComponent(method);
        	formDataValues.argJSON = encodeURIComponent(argumentValue);
        	
        	var requstUrl = PortalTools.callEntityMethodURL + "?" + qs.join("&");
        	var request = $.post(requstUrl, formDataValues);
        	
        	// If a progress bar is supplied, setup polling to keep it updated.
	        if(uiProgressBar != null) {
	            PortalTools.monitorRemoteMethod(method, argumentValue, timestamp, uiProgressBar, spinnerOnly);
	        }
        	
        	request.success(function() {
	            if(typeof timer !== "undefined" && typeof beginTimeoutCountdown === "function") {
	                beginTimeoutCountdown();
	            }
	        });
        	
        	return request;
		};
		
		return returnObj;
	}

	// Used to make an asynchronous call to a server side EType method
	// methodName: the method to call, example: CustomRemoteMethods.myMethod
	// arg: a string, passed as the only argument to the method above
	// uiProgressBar: a jQuery ui element which will be replaced with a progress indicator
	// spinnerOnly: set to true to prevent polling, the progress bar is a spinner instead
    PortalTools.callRemoteMethod = function(methodName, arg, uiProgressBar, spinnerOnly) {
    	return PortalTools.RemoteMethod(methodName)
    		.Argument(arg)
    		.ProgressBar(uiProgressBar, spinnerOnly)
    		.Call();
    };

    // Because we need to maintain access to information about the original call, this function
    // makes heavy use of $.proxy() to retain the calling context
    PortalTools.monitorRemoteMethod = function(methodName, arg, timestamp, uiProgressBar, spinnerOnly) {
        // validate arguments
        if(typeof(methodName) !== "string") {
            throw new Error("PortalTools.monitorRemoteMethod(): invalid methodName");
        }
        if(typeof(arg) !== "string") {
            throw new Error("PortalTools.monitorRemoteMethod(): invalid arg");
        }
        if(typeof(timestamp) !== "string" && typeof(timestamp) !== "number") {
            throw new Error("PortalTools.monitorRemoteMethod(): invalid timestamp");
        }
        try{
            var tagName = uiProgressBar.prop("tagName");
            if(tagName !== "SPAN" && tagName !== "DIV") {
                throw new Error("PortalTools.monitorRemoteMethod(): invalid uiProgressBar");
            }
        } catch(e) {
            throw new Error("PortalTools.monitorRemoteMethod(): invalid uiProgressBar, " + e.description);
        }

        // override progress bar's background to always be white
        uiProgressBar.css("background-color", "#FFFFFF");

        // build query string for monitor GET
        var qsMon = [];
        qsMon.push("method=" + encodeURIComponent(methodName));
        qsMon.push("argJSON=" + encodeURIComponent(arg));
        qsMon.push("time=" + encodeURIComponent(timestamp));
        var qsMonString = qsMon.join("&");

        // initialize state, the "progress bar" should just be a spinner for the first
        // interval when no querying is occurring
        var context = this;
        try {
            // attempt to destroy any pre-existing progress bar, there is no way to detect this
            //This if check is to address the issue in IE8(TFS Bug # 3769). Can rid of it in 7.0
            if(uiProgressBar.hasClass("ui-progressbar")){
            	uiProgressBar.progressbar("destroy");
            }
        } catch(e) {
            //ignore exception
        }
        uiProgressBar.html($("<img src='" + PortalTools.busyGifPath + "' />"));
        var displayingSpinner = true;

        // this method is called to initiate a monitor GET
        var innerMRM = function() {
            // We're polling now, so convert spinner to progress bar.
            // Polling won't work in some cases, like on Webkit browsers after a form submit (which we rely on
            // for activity submission) so set value to "false" initially, which is the 'indeterminate' mode.
            // If polling does work, it will get set to a number.
            if(displayingSpinner && spinnerOnly !== true) {
                uiProgressBar.html("");
                try {
                    // attempt to destroy any pre-existing progress bar, there is no way to detect this
                    uiProgressBar.progressbar("destroy");
                } catch(e) {
                    //ignore exception
                }
                uiProgressBar.progressbar({ value: false });
                displayingSpinner = false;
            }

            if(!displayingSpinner) {
                var request = $.get(PortalTools.monitorEntityMethodURL + "?" + qsMonString);
                request.then(handlerMRM);
            }
        };



        // this method receives the result from the server
        var handlerMRM = function(data) {
            var result = JSON.parse(data);
            var value = parseInt(result.percentComplete, 10);
            var stop;

            if(value > 0 && value <= 100) {
                if(result.message === null) {
                    uiProgressBar
                        .progressbar( { 'value': value } );
                } else {
                    uiProgressBar
                        .progressbar( { 'value': value } )
                        .children('.ui-progressbar-value')
                        .html($('<p style="white-space:nowrap; position:absolute;">' + result.message + "</p>"));
                }
                stop = value >= 100 || value < 0;
                if(!stop) {
                    setTimeout(
                        $.proxy(innerMRM, context),
                        PortalTools.remoteMethodMonitorPollInterval
                    );
                }
            } else {
                uiProgressBar.progressbar( "value" , 0 );
            }
        };

        setTimeout(
            $.proxy(innerMRM, context),
            PortalTools.remoteMethodMonitorFirstPollDelay
        );
    };

    PortalTools.loadLayoutIntoTag = function (layout, queryString, tagID) {
        layout = layout.replace(/\:/g, "");
        var url = PortalTools.urlPrefix + layout + "?" + queryString;
        var tagObject = $("#" + tagID);
        tagObject.fadeTo(0, 0.3);
        var request = $.get(url);
        request.success(function(data) {
            var body;

            var unencodedURL = 'var _webrUnencodedUrl=\"' + PortalTools.urlPrefix + layout;

            if(data.indexOf(unencodedURL) !== -1) {
                // we only want to insert the body portion of the page, but if that fails, just
                // plug it all in and hope for the best
                try {
                    var bodyStartRegex = new RegExp("(.*<\\s*body[^>]*>)");
                    var bodyStart = data.match(bodyStartRegex)[0];
                    var bodyStartIndex = data.indexOf(bodyStart) + bodyStart.length;
                    var bodyEndIndex = data.lastIndexOf("</body>");
                    body = data.substring(bodyStartIndex, bodyEndIndex);
                } catch(e) {
                    body = data;
                }
                tagObject.html(body);
                tagObject.fadeTo(0, 1);
                document.dispatchEvent(PortalTools.AsyncLoadEvent);
            } else {
                // a redirect probably occurred, meaning the session probably expired
                tagObject.html("Access denied");
                tagObject.fadeTo(0, 1);
            }
        });

        return request;
    };

	PortalTools.renderCalendarSquareDate = function(date, warningLevel) {
		if(date == null) {
			return "<div class='CalendarSquareDate' title='No date specified' aria-label='No date specified'>" +
			       "<div class='NoDate' aria-hidden='true'>?</div>" + 
			       "</div>";
		}
	
		var classes = ["CalendarSquareDate"];
		if(typeof warningLevel === "string") {
			if(warningLevel === "overdue") classes.push("Overdue");
			else if(warningLevel === "warning") classes.push("Warning");
		}
		var month = date.toLocaleString("en-es", { month: "short" });
		var ariaDate = moment(date).format("MMMM Do");
		var title =  moment(date).format("ll");
		var isoString = date.toISOString();
		return "<time class=\"" + classes.join(" ") + "\" datetime=\"" + isoString + "\" aria-label='" + ariaDate + "' title='" + title + "'>" +
		       "<span class='calendarMonth' aria-hidden='true'>" + month + "</span><span class='calendarDay' aria-hidden='true'>" + date.getDate() + "</span></time>";
	};

	PortalTools.getQualifiedAttribute = function(rootObj, path) {
		var arrPath = path.split(".");
		var i = 0;
		var obj = rootObj;
		while(i < arrPath.length) {
			obj = obj[arrPath[i]];
			if(obj == null || typeof obj == "undefined") return obj;
			i++;
		}
		if(typeof obj === "undefined") return null;
		return obj;
	};

	PortalTools.compactString = function(str, length) {
		// strings are allowed, null or undefined treated as empty string			
		var strValue = 
			typeof str == "string" ? str :
			str == null || typeof str == "undefined" ? "" :
			null; // indicates error value
			
		if(strValue == null) {
			throw new Error("compactString() must be passed a string argument");
		}
		
		var len = typeof length == "number" && length > 0 ? length : 255;
		
		if(strValue.length <= len)
			return strValue;
		
		var segmentLength = Math.floor((len-1) / 2);
		var first = strValue.substring(0, segmentLength);
		var last = strValue.substring(strValue.length-segmentLength);
		return first + "..." + last;
	};

	PortalTools.extractOidHexString = function(oid) {
		if(typeof oid !== "string") throw new Error("PortalTools.extractOidHexString() must be a passed a string");
   		var re = /\[OID\[(\w{32})\]\]/;
		return oid.match(re)[1];
	};

	PortalTools.htmlEncode = function(x) {
		return String(x)
	        .replace(/&/g, '&amp;')
	        .replace(/\"/g, '&quot;')
	        .replace(/\'/g, '&#39;')
	        .replace(/</g, '&lt;')
	        .replace(/>/g, '&gt;')
	        .replace(/\//g, '&#x2F;');
	};
	
	PortalTools.htmlDecode = function(x) {
	    return String(x)
	        .replace(/&quot;/g, '"')
	        .replace(/&#39;/g, "'")
	        .replace(/&lt;/g, '<')
	        .replace(/&gt;/g, '>')
	        .replace(/&amp;/g, '&')
	        .replace(/&#x2f;/g, '\/');
	};	

	PortalTools.AsyncLoadEvent = document.createEvent("Event");
	PortalTools.AsyncLoadEvent.initEvent("PortalTools.AsyncLoadEvent", true, true);

	PortalTools.ShowDropLayout = function(ele) {
		var element = $(ele);
		element.attr("aria-expanded", "true");
		var contentSelector = element.data("contentSelector");
		var content = $(contentSelector);
		content.css("min-width", element.width() + "px");
		
		//force the droplayout to never be bigger than the window, which will 
		//show a scrollbar on the droplayout rather than require scrolling the window
		var windowHeight = window.innerHeight - 20;  //give is a 20px buffer for scrollbars and such.
		content.css("max-height", windowHeight + "px");
		
		// if necessary, load content dynamically
		var dynamicContentUrl = content.data("url");
		var dynamicContentQs = content.data("queryString");
		var alreadyLoaded = content.data("alreadyLoaded");
		if(dynamicContentUrl != null && dynamicContentUrl != "" && alreadyLoaded != "loaded" ) {
			PortalTools.loadLayoutIntoTag(dynamicContentUrl, dynamicContentQs, content.attr("id")).success(function() {
				content.data("alreadyLoaded", "loaded");
				content.position({my: "left top", at: "left bottom", of: element, collision: "flipfit"});
			});
		}
		
		// show the content
		content.show(0, function() {
			content.position({
				my: "left top", at: "left bottom", of: element, collision: "flipfit"
			});
		});
		
		// since the link and content span might have been loaded dynamically (independent
		// of the content within the span the might load dynamically) we attach the mouseout
		// handlers here.
		var closeDropLayout = function() {
			if( !element.is(":hover") && !content.is(":hover") ) {
				content.hide();
				content.off("mouseout");
				element.off("mouseout");
				$(window).unbind("touchstart", closeDropLayout);
				element.attr("aria-expanded", "false");
			}
		};
		
		// in the mouse out case, closing it immediately is too abrupt
		var mouseOutHandler = function() {
			setTimeout(closeDropLayout, 1000);
		};
		
		content.on("mouseout", mouseOutHandler);
		element.on("mouseout", mouseOutHandler);
		$(window).on("touchstart", closeDropLayout);
		
		
		return false;
	};

	// There are strange interactions between the main elements sizing, DRSVs,
	// and responsive tabs.  Calling this adjusts the top bar size to something
	// sane, before tabs size themselves relative to it.  Also prevents weird
	// top bar widths when the page is resized, either manually or due to a 
	// DRSV contents load stretching the document width.
	PortalTools.AdjustTopBarWidth = function() {
		var jqMain = $("main");
		if(jqMain.length == 0) return;
		
		// It is necessary to hide all tabs while the top bar width is calculated,
		// or a large number of tabs will stretch the top bar.
		$(".NavigationTabControl").hide();
        var margin = jqMain.offset().left + 1;
        var topBarWidth = jqMain.width() + (margin * 2);
        $("table#ClickTopBar").width(""+topBarWidth+"px");
        $(".NavigationTabControl").show();
	}

	$(window).resize(_.debounce(PortalTools.AdjustTopBarWidth));

    PortalTools.deserializeCLRDateJSON = function(dateStr) {
        return new Date(parseInt(dateStr.substr(6), 10));
    };

    PortalTools.getTabQS = function() {
        var qsArray = [];
        _.each(PortalTools.tabState, function(element, index) {
            qsArray.push(index + "=" + encodeURIComponent(element));
        });
        return qsArray.join("&");
    };

    PortalTools.EventType = function() {
        this.successHandlers = [];
        this.failureHandlers = [];

        this.then = function(_func) {
            if(typeof _func !== "function") {
                throw new Error("argument must be a function");
            }
            this.successHandlers.push(_func);
            this.failureHandlers.push(_func);
        };

        this.done = function(_func) {
            if(typeof _func !== "function") {
                throw new Error("argument must be a function");
            }
            this.successHandlers.push(_func);
        };

        this.fail = function(_func) {
            if(typeof _func !== "function") {
                throw new Error("argument must be a function");
            }
            this.failureHandlers.push(_func);
        };

        this.prepare = function() {
            var x = new $.Deferred();
            var i;
            for(i in this.successHandlers) {
                if(this.successHandlers.hasOwnProperty(i)) {
                    x.done(this.successHandlers[i]);
                }
            }
            for(i in this.failureHandlers) {
                if(this.failureHandlers.hasOwnProperty(i)) {
                    x.fail(this.failureHandlers[i]);
                }
            }
            return x;
        };

        return this;
    };

    PortalTools.createCookie = function(name,value,days,path,domain,secure) {
        var expires;
        if (days) {
            var date = new Date();
            date.setTime(date.getTime()+(days*24*60*60*1000));
            expires = "; expires="+date.toGMTString();
        }

        document.cookie = name + "=" + encodeURIComponent(value) +
        ((expires) ? "; expires=" + expires : "") +
        ((path)    ? "; path="    + path    : ";path=/") +
        ((domain)  ? "; domain="  + domain  : "") +
        ((secure)  ? "; secure"             : "");
    };

    PortalTools.readCookie = function(name) {
        var nameEQ = name + "=";
        var ca = document.cookie.split(';');
        for(var i=0;i < ca.length;i++) {
            var c = ca[i];
            while (c.charAt(0)===' ') {
                c = c.substring(1,c.length);
            }
            if (c.indexOf(nameEQ) === 0) {
                return c.substring(nameEQ.length,c.length);
            }
        }
        return null;
    };

    PortalTools.eraseCookie = function(name) {
        PortalTools.createCookie(name,"",-1);
    };

    // jQuery utility function
    // jQuery selectors cannot have unescaped periods, this escapes them.
    PortalTools.escapeJQuerySelector = function(str) {
        return str.replace(/(:|\.|\[|\]|\(|\))/g,"\\$1");
    };
    
    var _webr_qs = {};
	var _qs = "";
	_initializeQS();
    
	function _initializeQS(){
		var rawLocation = _webrUnencodedUrl ? _webrUnencodedUrl : window.location;
		var location = rawLocation.replace(/#.*$/g, ""); // remove hash from end of url
		var qsStart = location.indexOf("?");
		if(qsStart === -1) return;
		_qs = location.substring(qsStart+1);
		var qsArray = _qs.split("&");
		
		var temp, i, idx;
		for(i=0; i<qsArray.length; i++) {
			try {
				temp = qsArray[i].split("=");
				idx = decodeURIComponent(temp[0]).toLowerCase();
				_webr_qs[idx]=decodeURIComponent(temp[1]);
			} catch(e) {
				// fail silently in the case of a badly formed query string
			}
		}
	}
	
    PortalTools.QueryString = function(id) {
        try {
            var x = _webr_qs[id.toLowerCase()];
            if(typeof x === "undefined") return null
            return x;
        } catch(e) {
            return null;
        }
    };

    PortalTools.GetEntireQueryString = function() {
        return _qs;
    };
    
    PortalTools.RecomputeQueryStringParams = function(){
    	_initializeQS();
    }

    $(function() {
    	//Initialization to adapt the new tab control on the page
		$("a.TabOn, span.TabOn").parent().addClass("TabOn");
		$("a.TabOff").each(function(){
			$(this).parent().addClass("TabOff");
		});
		$("td.TabOn").closest('table').addClass("NavigationTabArea");
	});
	
	PortalTools.LoadedScriptFiles = {};
	
	PortalTools.HasScriptFileBeenLoaded = function(strLibName) {
		if(typeof PortalTools.LoadedScriptFiles[strLibName] !== "undefined") {
			return true;
		}
		
		PortalTools.LoadedScriptFiles[strLibName] = true;
		return false;
	};
	
})();

